from audio import Audio

from mini.track.dialog import Message
from mini.track.dialog import Speaker
from owg.util.opengl import ColorF

from mini.track import TrackSpawner
from mini.track import EventKeys
from mini.track.fx import Earthquake

hacker_model = world["HackerModel"]

speaker = Speaker(v3d(hacker_model).asV3F().add(0, 0, 2), ColorF("#75C3FF"), "Hacker")
speaker.voice = Audio.getClip("hacker speak")

hacking_sound = Audio.getClip("hacking activity")
hacking_sound.setPosition(v3d(hacker_model).p, 1)
hacking_sound.setLooping(True)

hacking_ball = world["HackingBallQuestItem"]

s = SimpleNamespace()
s.scene_enqueued = False

pitches = [261.63, 277.18, 293.66, 311.13, 329.63, 349.23, 369.99, 392, 415.3, 440, 466.16]
s.current_pitch_index = 0

def play_orchestra_hit():
    rate = pitches[s.current_pitch_index] / pitches[0]
    Audio.play("orchestra hit", None, 0.9, None, rate)
    s.current_pitch_index += 1

def message(text, life_in_frames):
    return Message(speaker, text, life_in_frames)

def speak(text, life_in_frames):
    game.dialogDisplay.showMessage(message(text, life_in_frames))

def has_hacked_atm():
    return EventKeys.has(world["UnlockTextbogusEvent"])

def onEnter():
    if has_hacked_atm():
        speak("Heh, I'm totally exhausted from that hacking. Punching in all those #FF00001s and 0s## (binary code) is no easy feat!", 210)
    elif not hacking_ball.isCollected():
        speak("A hacker like myself is useless without their #FF0000hacking ball##! I think I abandoned it in the park somewhere.", 210)
    elif not s.scene_enqueued:
        enqueue_cutscene(script)
        s.scene_enqueued = True

def script(ctx):
    world["HackerFacePlayer"].enabled = False
    game.player.pos.set(world["HackingPutPlayer"].pos.asV2D())
    game.player.angle = 4
    hacker_model.angle = -1

    ctx.camera.lookAt(world["HackingFrom"], world["HackingTo"])
    ctx.sleep(15)
    ctx.show(message("Woah, that's my #FF0000hacking ball##! Let me use that to hack this ATM.", 90)).await()
    hacking_ball.give(hacker_model)
    ctx.sleep(45)

    ctx.camera.lookAt(world["HackingTo"], v3d(hacker_model).z(3))
    hacker_model.angle = -.8
    ctx.sleep(15)
    ctx.show(message("I need total silence.", 60)).await()
    ctx.sleep(15)
    game.musicPlayer.stop()
    ctx.sleep(45)

    earthquake = Earthquake([world["AtmModel"]])
    hacking_sound.play()
    ctx.camera.lookAt(world["HackingFrom2"], world["TextbogusTo"])
    hacker_model.angle = 4 - 1.5

    # TODO sounds and cute fx...
    ctx.show(message("This ATM's security system has a fatal flaw. Let me just inject some malicious code...", 210)).await()
    ctx.move(ctx.camera.eye).to(world["HackingFrom3"]).now()

    play_orchestra_hit()
    ctx.show(message("Ooh, tricky-tricky! I wasn't expecting that... I'll have to use a separate back door...", 140)).await()
    ctx.move(ctx.camera.eye).to(world["HackingFrom4"]).now()

    play_orchestra_hit()
    ctx.show(message("Alright, I'm in!", 40)).await()
    ctx.move(ctx.camera.eye).to(world["HackingFrom5"]).now()

    play_orchestra_hit()
    ctx.show(message("OK, now I'm browsing some open source software on the dark web's NPM...", 70)).await()
    ctx.move(ctx.camera.eye).to(world["HackingFrom6"]).now()

    play_orchestra_hit()
    ctx.show(message("Cloning the software via the Git command line interface...", 50)).await()

    earthquake.destroy()
    hacking_sound.stop()
    game.musicPlayer.start()
    ctx.camera.lookAt(v3d(world["HackerCloseup"]).z(3), v3d(hacker_model).z(3))
    play_orchestra_hit()
    ctx.show(message("And... !", 20)).await()

    world["HackerFacePlayer"].enabled = True

def onLeave():
    pass